@@ -165,6 +165,19 @@ clutter_clone_texture_paint (ClutterActor *self)
   if (!CLUTTER_ACTOR_IS_REALIZED (parent_texture))
     clutter_actor_realize (parent_texture);
 
+  /* If 'parent' texture isn't visible we run its paint to be sure it 
+   * updates. Needed for TFP and likely FBOs. 
+   * Potentially could cause issues 
+   */
+  if (!clutter_actor_get_paint_visibility(parent_texture))
+    {
+      CLUTTER_SET_PRIVATE_FLAGS(parent_texture,
+                                CLUTTER_TEXTURE_IN_CLONE_PAINT);
+      g_signal_emit_by_name (priv->parent_texture, "paint", NULL);
+      CLUTTER_UNSET_PRIVATE_FLAGS(parent_texture,
+                                  CLUTTER_TEXTURE_IN_CLONE_PAINT);
+    }
+
   col.alpha = clutter_actor_get_paint_opacity (self);
   cogl_color (&col);
 
@@ -209,6 +222,9 @@ set_parent_texture (ClutterCloneTexture *ctexture,
   ClutterActor *actor = CLUTTER_ACTOR (ctexture);
   gboolean was_visible = CLUTTER_ACTOR_IS_VISIBLE (ctexture);
 
+  if (priv->parent_texture == texture)
+    return;
+
   if (priv->parent_texture)
     {
       g_object_unref (priv->parent_texture);
