@@ -505,30 +505,17 @@ traverse_children (GNode    *node,
     case REMOVE_BY_ID:
       if (closure->d.id == entry->id)
         {
-          if (entry->complete_id)
-            {
-              g_signal_handler_disconnect (entry->timeline, entry->complete_id);
-              entry->complete_id = 0;
-            }
-
-          if (entry->marker_id)
-            {
-              g_signal_handler_disconnect (entry->timeline, entry->marker_id);
-              entry->marker_id = 0;
-            }
-
-          g_object_unref (entry->timeline);
-
+          /* Destroy all the child entries of this node */
           g_node_traverse (node,
                            G_POST_ORDER,
                            G_TRAVERSE_ALL,
                            -1,
                            destroy_entry, NULL);
 
-          g_free (entry->marker);
-          g_slice_free (ClutterScoreEntry, entry);
-
+          /* Keep track of this node so that it will be destroyed
+             further up */
           closure->result = node;
+
           retval = TRUE;
         }
       break;
@@ -599,7 +586,12 @@ start_children_entries (GNode    *node,
 {
   ClutterScoreEntry *entry = node->data;
 
-  start_entry (entry);
+  /* If data is NULL, start all entries that have no marker, otherwise
+     only start entries that have the same marker */
+  if (data == NULL
+      ? entry->marker == NULL
+      : (entry->marker && !strcmp (data, entry->marker)))
+    start_entry (entry);
 }
 
 static void
@@ -623,7 +615,7 @@ on_timeline_marker (ClutterTimeline   *timeline,
       g_node_children_foreach (parent,
                                G_TRAVERSE_ALL,
                                start_children_entries,
-                               NULL);
+                               (gpointer) marker_name);
     }
 }
 
@@ -765,6 +757,7 @@ clutter_score_start (ClutterScore *score)
     }
   else
     {
+      g_signal_emit (score, score_signals[STARTED], 0);
       g_node_children_foreach (priv->root,
                                G_TRAVERSE_ALL,
                                start_children_entries,
