@@ -1373,9 +1373,6 @@ clutter_timeline_get_progressx (ClutterTimeline *timeline)
   progress = clutter_qdivx (CLUTTER_INT_TO_FIXED (priv->current_frame_num),
 			    CLUTTER_INT_TO_FIXED (priv->n_frames));
 
-  if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
-    progress = CFX_ONE - progress;
-
   return progress;
 }
 
