@@ -54,8 +54,9 @@ enum
     COGL_GLES2_DIRTY_FOG_END          = 1 << 5,
     COGL_GLES2_DIRTY_FOG_COLOR        = 1 << 6,
     COGL_GLES2_DIRTY_ALPHA_TEST_REF   = 1 << 7,
+    COGL_GLES2_DIRTY_TEXTURE_UNIT     = 1 << 8,
 
-    COGL_GLES2_DIRTY_ALL              = (1 << 8) - 1
+    COGL_GLES2_DIRTY_ALL              = (1 << 9) - 1
   };
 
 struct _CoglGles2WrapperUniforms
@@ -71,6 +72,8 @@ struct _CoglGles2WrapperUniforms
   GLint     fog_color_uniform;
 
   GLint     alpha_test_ref_uniform;
+
+  GLint     texture_unit_uniform;
 };
 
 struct _CoglGles2WrapperSettings
@@ -242,7 +245,8 @@ void cogl_wrap_glFogx (GLenum pname, GLfixed param);
 void cogl_wrap_glFogxv (GLenum pname, const GLfixed *params);
 
 void cogl_wrap_glDrawArrays (GLenum mode, GLint first, GLsizei count);
-
+void cogl_wrap_glDrawElements (GLenum mode, GLsizei count, GLenum type,
+                               const GLvoid *indices);
 void cogl_wrap_glTexParameteri (GLenum target, GLenum pname, GLfloat param);
 
 void cogl_gles2_wrapper_bind_texture (GLenum target, GLuint texture,
@@ -266,6 +270,7 @@ void _cogl_gles2_clear_cache_for_program (CoglHandle program);
 
 #define cogl_wrap_glClearColorx        glClearColorx
 #define cogl_wrap_glDrawArrays         glDrawArrays
+#define cogl_wrap_glDrawElements       glDrawElements
 #define cogl_wrap_glPushMatrix         glPushMatrix
 #define cogl_wrap_glPopMatrix          glPopMatrix
 #define cogl_wrap_glMatrixMode         glMatrixMode
