@@ -174,6 +174,35 @@ _cogl_span_iter_end (CoglSpanIter *iter)
   return iter->pos >= iter->cover_end;
 }
 
+static guchar *
+_cogl_texture_allocate_waste_buffer (CoglTexture *tex)
+{
+  CoglTexSliceSpan *last_x_span;
+  CoglTexSliceSpan *last_y_span;
+  guchar           *waste_buf = NULL;
+
+  /* If the texture has any waste then allocate a buffer big enough to
+     fill the gaps */
+  last_x_span = &g_array_index (tex->slice_x_spans, CoglTexSliceSpan,
+                                tex->slice_x_spans->len - 1);
+  last_y_span = &g_array_index (tex->slice_y_spans, CoglTexSliceSpan,
+                                tex->slice_y_spans->len - 1);
+  if (last_x_span->waste > 0 || last_y_span->waste > 0)
+    {
+      gint bpp = _cogl_get_format_bpp (tex->bitmap.format);
+      CoglTexSliceSpan  *first_x_span
+        = &g_array_index (tex->slice_x_spans, CoglTexSliceSpan, 0);
+      CoglTexSliceSpan  *first_y_span
+        = &g_array_index (tex->slice_y_spans, CoglTexSliceSpan, 0);
+      guint right_size = first_y_span->size * last_x_span->waste;
+      guint bottom_size = first_x_span->size * last_y_span->waste;
+
+      waste_buf = g_malloc (MAX (right_size, bottom_size) * bpp);
+    }
+
+  return waste_buf;
+}
+
 static gboolean
 _cogl_texture_upload_to_gl (CoglTexture *tex)
 {
@@ -182,6 +211,7 @@ _cogl_texture_upload_to_gl (CoglTexture *tex)
   GLuint             gl_handle;
   gint               bpp;
   gint               x,y;
+  guchar            *waste_buf;
   CoglBitmap         slice_bmp;
   
   /* FIXME: might optimize by not copying to intermediate slice
@@ -190,6 +220,8 @@ _cogl_texture_upload_to_gl (CoglTexture *tex)
   
   bpp = _cogl_get_format_bpp (tex->bitmap.format);
   
+  waste_buf = _cogl_texture_allocate_waste_buffer (tex);
+
   /* Iterate vertical slices */
   for (y = 0; y < tex->slice_y_spans->len; ++y)
     {
@@ -233,6 +265,62 @@ _cogl_texture_upload_to_gl (CoglTexture *tex)
 			       tex->gl_format, tex->gl_type,
 			       slice_bmp.data) );
 
+          /* Fill the waste with a copies of the rightmost pixels */
+          if (x_span->waste > 0)
+            {
+              const guchar *src = tex->bitmap.data
+                + y_span->start * tex->bitmap.rowstride
+                + (x_span->start + x_span->size - x_span->waste - 1) * bpp;
+              guchar *dst = waste_buf;
+              guint wx, wy;
+
+              for (wy = 0; wy < y_span->size - y_span->waste; wy++)
+                {
+                  for (wx = 0; wx < x_span->waste; wx++)
+                    {
+                      memcpy (dst, src, bpp);
+                      dst += bpp;
+                    }
+                  src += tex->bitmap.rowstride;
+                }
+
+              GE( glTexSubImage2D (tex->gl_target, 0,
+                                   x_span->size - x_span->waste, 0,
+                                   x_span->waste,
+                                   y_span->size - y_span->waste,
+                                   tex->gl_format, tex->gl_type,
+                                   waste_buf) );
+            }
+
+          if (y_span->waste > 0)
+            {
+              const guchar *src = tex->bitmap.data
+                + ((y_span->start + y_span->size - y_span->waste - 1)
+                   * tex->bitmap.rowstride)
+                + x_span->start * bpp;
+              guchar *dst = waste_buf;
+              guint wy, wx;
+
+              for (wy = 0; wy < y_span->waste; wy++)
+                {
+                  memcpy (dst, src, (x_span->size - x_span->waste) * bpp);
+                  dst += (x_span->size - x_span->waste) * bpp;
+
+                  for (wx = 0; wx < x_span->waste; wx++)
+                    {
+                      memcpy (dst, dst - bpp, bpp);
+                      dst += bpp;
+                    }
+                }
+
+              GE( glTexSubImage2D (tex->gl_target, 0,
+                                   0, y_span->size - y_span->waste,
+                                   x_span->size,
+                                   y_span->waste,
+                                   tex->gl_format, tex->gl_type,
+                                   waste_buf) );
+            }
+
 	  if (tex->auto_mipmap)
 	    cogl_wrap_glGenerateMipmap (tex->gl_target);
 	  
@@ -240,7 +328,10 @@ _cogl_texture_upload_to_gl (CoglTexture *tex)
 	  g_free (slice_bmp.data);
 	}
     }
-  
+
+  if (waste_buf)
+    g_free (waste_buf);
+
   return TRUE;
 }
 
@@ -494,17 +585,22 @@ _cogl_texture_upload_subregion_to_gl (CoglTexture *tex,
 				      GLuint       source_gl_format,
 				      GLuint       source_gl_type)
 {
-  gint         bpp;
-  CoglSpanIter x_iter;
-  CoglSpanIter y_iter;
-  GLuint       gl_handle;
-  gint         source_x = 0, source_y = 0;
-  gint         inter_w = 0, inter_h = 0;
-  gint         local_x = 0, local_y = 0;
-  CoglBitmap   slice_bmp;
+  CoglTexSliceSpan *x_span;
+  CoglTexSliceSpan *y_span;
+  gint              bpp;
+  CoglSpanIter      x_iter;
+  CoglSpanIter      y_iter;
+  GLuint            gl_handle;
+  gint              source_x = 0, source_y = 0;
+  gint              inter_w = 0, inter_h = 0;
+  gint              local_x = 0, local_y = 0;
+  guchar           *waste_buf;
+  CoglBitmap        slice_bmp;
   
   bpp = _cogl_get_format_bpp (source_bmp->format);
-  
+
+  waste_buf = _cogl_texture_allocate_waste_buffer (tex);
+
   /* FIXME: might optimize by not copying to intermediate slice
      bitmap when source rowstride = bpp * width and the texture
      image is not sliced */
@@ -526,7 +622,10 @@ _cogl_texture_upload_subregion_to_gl (CoglTexture *tex,
 	  inter_h = 0;
 	  continue;
 	}
-      
+
+      y_span = &g_array_index (tex->slice_y_spans, CoglTexSliceSpan,
+                               y_iter.index);
+
       /* Iterate horizontal spans */
       for (source_x = src_x,
 	   _cogl_span_iter_begin (&x_iter, tex->slice_x_spans,
@@ -544,7 +643,10 @@ _cogl_texture_upload_subregion_to_gl (CoglTexture *tex,
 	      inter_w = 0;
 	      continue;
 	    }
-	  
+
+          x_span = &g_array_index (tex->slice_x_spans, CoglTexSliceSpan,
+                                   x_iter.index);
+
 	  /* Pick intersection width and height */
 	  inter_w = CLUTTER_FIXED_TO_INT (x_iter.intersect_end -
 					  x_iter.intersect_start);
@@ -591,7 +693,81 @@ _cogl_texture_upload_subregion_to_gl (CoglTexture *tex,
 			       source_gl_format,
 			       source_gl_type,
 			       slice_bmp.data) );
-	  
+
+          /* If the x_span is sliced and the upload touches the
+             rightmost pixels then fill the waste with copies of the
+             pixels */
+          if (x_span->waste > 0
+              && local_x < x_span->size - x_span->waste
+              && local_x + inter_w >= x_span->size - x_span->waste)
+            {
+              const guchar *src = source_bmp->data
+                + (src_y + CLUTTER_FIXED_TO_INT (y_iter.intersect_start)
+                   - dst_y) * source_bmp->rowstride
+                + (src_x + x_span->start + x_span->size - x_span->waste
+                   - dst_x - 1) * bpp;
+              guchar *dst = waste_buf;
+              guint wx, wy;
+
+              for (wy = 0; wy < inter_h; wy++)
+                {
+                  for (wx = 0; wx < x_span->waste; wx++)
+                    {
+                      memcpy (dst, src, bpp);
+                      dst += bpp;
+                    }
+                  src += source_bmp->rowstride;
+                }
+
+              GE( glTexSubImage2D (tex->gl_target, 0,
+                                   x_span->size - x_span->waste, local_y,
+                                   x_span->waste,
+                                   inter_h,
+                                   source_gl_format,
+                                   source_gl_type,
+                                   waste_buf) );
+            }
+
+          /* same for the bottom-most pixels */
+          if (y_span->waste > 0
+              && local_y < y_span->size - y_span->waste
+              && local_y + inter_h >= y_span->size - y_span->waste)
+            {
+              const guchar *src = source_bmp->data
+                + (src_x + CLUTTER_FIXED_TO_INT (x_iter.intersect_start)
+                   - dst_x) * bpp
+                + (src_y + y_span->start + y_span->size - y_span->waste
+                   - dst_y - 1) * source_bmp->rowstride;
+              guchar *dst = waste_buf;
+              guint wy, wx;
+              guint copy_width;
+
+              if (local_x + inter_w >= x_span->size - x_span->waste)
+                copy_width = x_span->size - local_x;
+              else
+                copy_width = inter_w;
+
+              for (wy = 0; wy < y_span->waste; wy++)
+                {
+                  memcpy (dst, src, inter_w * bpp);
+                  dst += inter_w * bpp;
+
+                  for (wx = inter_w; wx < copy_width; wx++)
+                    {
+                      memcpy (dst, dst - bpp, bpp);
+                      dst += bpp;
+                    }
+                }
+
+              GE( glTexSubImage2D (tex->gl_target, 0,
+                                   local_x, y_span->size - y_span->waste,
+                                   copy_width,
+                                   y_span->waste,
+                                   source_gl_format,
+                                   source_gl_type,
+                                   waste_buf) );
+            }
+
 	  if (tex->auto_mipmap)
 	    cogl_wrap_glGenerateMipmap (tex->gl_target);
 
@@ -599,7 +775,10 @@ _cogl_texture_upload_subregion_to_gl (CoglTexture *tex,
 	  g_free (slice_bmp.data);
 	}
     }
-  
+
+  if (waste_buf)
+    g_free (waste_buf);
+
   return TRUE;
 }
 
@@ -1705,6 +1884,97 @@ cogl_texture_get_data (CoglHandle       handle,
   return byte_size;
 }
 
+static void
+_cogl_texture_flush_vertices (void)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (ctx->texture_vertices->len > 0)
+    {
+      int needed_indices;
+      CoglTextureGLVertex *p
+        = (CoglTextureGLVertex *) ctx->texture_vertices->data;
+
+      /* The indices are always the same sequence regardless of the
+         vertices so we only need to change it if there are more
+         vertices than ever before */
+      needed_indices = ctx->texture_vertices->len / 4 * 6;
+      if (needed_indices > ctx->texture_indices->len)
+        {
+          int old_len = ctx->texture_indices->len;
+          int vert_num = old_len / 6 * 4;
+          int i;
+          GLushort *q;
+
+          /* Add two triangles for each quad to the list of
+             indices. That makes six new indices but two of the
+             vertices in the triangles are shared. */
+          g_array_set_size (ctx->texture_indices, needed_indices);
+          q = &g_array_index (ctx->texture_indices, GLushort, old_len);
+
+          for (i = old_len;
+               i < ctx->texture_indices->len;
+               i += 6, vert_num += 4)
+            {
+              *(q++) = vert_num + 0;
+              *(q++) = vert_num + 1;
+              *(q++) = vert_num + 3;
+
+              *(q++) = vert_num + 1;
+              *(q++) = vert_num + 2;
+              *(q++) = vert_num + 3;
+            }
+        }
+
+      GE( cogl_wrap_glVertexPointer (2, GL_FLOAT,
+                                     sizeof (CoglTextureGLVertex), p->v ) );
+      GE( cogl_wrap_glTexCoordPointer (2, GL_FLOAT,
+                                       sizeof (CoglTextureGLVertex), p->t ) );
+
+      GE( cogl_gles2_wrapper_bind_texture (ctx->texture_target,
+                                           ctx->texture_current,
+                                           ctx->texture_format) );
+
+      GE( cogl_wrap_glDrawElements (GL_TRIANGLES,
+                                    needed_indices,
+                                    GL_UNSIGNED_SHORT,
+                                    ctx->texture_indices->data) );
+
+      g_array_set_size (ctx->texture_vertices, 0);
+    }
+}
+
+static void
+_cogl_texture_add_quad_vertices (GLfloat x1, GLfloat y1,
+                                 GLfloat x2, GLfloat y2,
+                                 GLfloat tx1, GLfloat ty1,
+                                 GLfloat tx2, GLfloat ty2)
+{
+  CoglTextureGLVertex *p;
+  GLushort first_vert;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* Add the four vertices of the quad to the list of queued
+     vertices */
+  first_vert = ctx->texture_vertices->len;
+  g_array_set_size (ctx->texture_vertices, first_vert + 4);
+  p = &g_array_index (ctx->texture_vertices, CoglTextureGLVertex, first_vert);
+
+  p->v[0] = x1;  p->v[1] = y1;
+  p->t[0] = tx1; p->t[1] = ty1;
+  p++;
+  p->v[0] = x1;  p->v[1] = y2;
+  p->t[0] = tx1; p->t[1] = ty2;
+  p++;
+  p->v[0] = x2;  p->v[1] = y2;
+  p->t[0] = tx2; p->t[1] = ty2;
+  p++;
+  p->v[0] = x2;  p->v[1] = y1;
+  p->t[0] = tx2; p->t[1] = ty1;
+  p++;
+}
+
 static void
 _cogl_texture_quad_sw (CoglTexture *tex,
 		       ClutterFixed x1,
@@ -1716,40 +1986,46 @@ _cogl_texture_quad_sw (CoglTexture *tex,
 		       ClutterFixed tx2,
 		       ClutterFixed ty2)
 {
-  CoglSpanIter       iter_x    ,  iter_y;
-  ClutterFixed       tw        ,  th;
-  ClutterFixed       tqx       ,  tqy;
-  ClutterFixed       first_tx  ,  first_ty;
-  ClutterFixed       first_qx  ,  first_qy;
-  ClutterFixed       slice_tx1 ,  slice_ty1;
-  ClutterFixed       slice_tx2 ,  slice_ty2;
-  ClutterFixed       slice_qx1 ,  slice_qy1;
-  ClutterFixed       slice_qx2 ,  slice_qy2;
-  GLfixed            tex_coords[8];
-  GLfixed            quad_coords[8];
-  GLuint             gl_handle;
-  gulong             enable_flags = (COGL_ENABLE_TEXTURE_2D
-                                    | COGL_ENABLE_VERTEX_ARRAY
-                                    | COGL_ENABLE_TEXCOORD_ARRAY);
-  
+  CoglSpanIter    iter_x    ,  iter_y;
+  ClutterFixed    tw        ,  th;
+  ClutterFixed    tqx       ,  tqy;
+  ClutterFixed    first_tx  ,  first_ty;
+  ClutterFixed    first_qx  ,  first_qy;
+  ClutterFixed    slice_tx1 ,  slice_ty1;
+  ClutterFixed    slice_tx2 ,  slice_ty2;
+  ClutterFixed    slice_qx1 ,  slice_qy1;
+  ClutterFixed    slice_qx2 ,  slice_qy2;
+  GLuint          gl_handle;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
 #if COGL_DEBUG
   printf("=== Drawing Tex Quad (Software Tiling Mode) ===\n");
 #endif
-  
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
-  /* Prepare GL state */
-  if (ctx->color_alpha < 255
-      || tex->bitmap.format & COGL_A_BIT)
+
+  /* If the texture coordinates are backwards then swap both the
+     geometry and texture coordinates so that the texture will be
+     flipped but we can still use the same algorithm to iterate the
+     slices */
+  if (tx2 < tx1)
     {
-      enable_flags |= COGL_ENABLE_BLEND;
+      ClutterFixed temp = x1;
+      x1 = x2;
+      x2 = temp;
+      temp = tx1;
+      tx1 = tx2;
+      tx2 = temp;
     }
-  
-  cogl_enable (enable_flags);
-  
-  GE( cogl_wrap_glTexCoordPointer (2, GL_FIXED, 0, tex_coords) );
-  GE( cogl_wrap_glVertexPointer   (2, GL_FIXED, 0, quad_coords) );
-  
+  if (ty2 < ty1)
+    {
+      ClutterFixed temp = y1;
+      y1 = y2;
+      y2 = temp;
+      temp = ty1;
+      ty1 = ty2;
+      ty2 = temp;
+    }
+
   /* Scale ratio from texture to quad widths */
   tw = CLUTTER_INT_TO_FIXED (tex->bitmap.width);
   th = CLUTTER_INT_TO_FIXED (tex->bitmap.height);
@@ -1789,7 +2065,7 @@ _cogl_texture_quad_sw (CoglTexture *tex,
       
       slice_qy2 = first_qy +
 	CFX_QMUL (iter_y.intersect_end - first_ty, tqy);
-      
+
       /* Localize slice texture coordinates */
       slice_ty1 = iter_y.intersect_start - iter_y.pos;
       slice_ty2 = iter_y.intersect_end - iter_y.pos;
@@ -1797,8 +2073,7 @@ _cogl_texture_quad_sw (CoglTexture *tex,
       /* Normalize texture coordinates to current slice */
       slice_ty1 /= iter_y.span->size;
       slice_ty2 /= iter_y.span->size;
-      
-      
+
       /* Iterate until whole quad width covered */
       for (_cogl_span_iter_begin (&iter_x, tex->slice_x_spans,
 				  first_tx, tx1, tx2) ;
@@ -1818,11 +2093,11 @@ _cogl_texture_quad_sw (CoglTexture *tex,
 	  /* Localize slice texture coordinates */
 	  slice_tx1 = iter_x.intersect_start - iter_x.pos;
 	  slice_tx2 = iter_x.intersect_end - iter_x.pos;
-	  
+
 	  /* Normalize texture coordinates to current slice */
 	  slice_tx1 /= iter_x.span->size;
 	  slice_tx2 /= iter_x.span->size;
-	  
+
 #if COGL_DEBUG
 	  printf("~~~~~ slice (%d,%d)\n", iter_x.index, iter_y.index);
 	  printf("qx1: %f\n", CLUTTER_FIXED_TO_FLOAT (slice_qx1));
@@ -1839,22 +2114,24 @@ _cogl_texture_quad_sw (CoglTexture *tex,
 	  gl_handle = g_array_index (tex->slice_gl_handles, GLuint,
 				     iter_y.index * iter_x.array->len +
 				     iter_x.index);
-	  
-	  GE( cogl_gles2_wrapper_bind_texture (tex->gl_target, gl_handle,
-					       tex->gl_intformat) );
-	  
-	  /* Draw textured quad */
-	  tex_coords[0] = slice_tx1; tex_coords[1] = slice_ty1;
-	  tex_coords[2] = slice_tx2; tex_coords[3] = slice_ty1;
-	  tex_coords[4] = slice_tx1; tex_coords[5] = slice_ty2;
-	  tex_coords[6] = slice_tx2; tex_coords[7] = slice_ty2;
-	  
-	  quad_coords[0] = slice_qx1; quad_coords[1] = slice_qy1;
-	  quad_coords[2] = slice_qx2; quad_coords[3] = slice_qy1;
-	  quad_coords[4] = slice_qx1; quad_coords[5] = slice_qy2;
-	  quad_coords[6] = slice_qx2; quad_coords[7] = slice_qy2;
-	  
-	  GE (cogl_wrap_glDrawArrays (GL_TRIANGLE_STRIP, 0, 4) );
+
+          /* If we're using a different texture from the one already queued
+             then flush the vertices */
+          if (ctx->texture_vertices->len > 0
+              && gl_handle != ctx->texture_current)
+            _cogl_texture_flush_vertices ();
+          ctx->texture_target = tex->gl_target;
+          ctx->texture_current = gl_handle;
+          ctx->texture_format = tex->gl_intformat;
+
+          _cogl_texture_add_quad_vertices (CLUTTER_FIXED_TO_FLOAT (slice_qx1),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_qy1),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_qx2),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_qy2),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_tx1),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_ty1),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_tx2),
+                                           CLUTTER_FIXED_TO_FLOAT (slice_ty2));
 	}
     }
 }
@@ -1870,38 +2147,28 @@ _cogl_texture_quad_hw (CoglTexture *tex,
 		       ClutterFixed tx2,
 		       ClutterFixed ty2)
 {
-  GLfixed           tex_coords[8];
-  GLfixed           quad_coords[8];
   GLuint            gl_handle;
   CoglTexSliceSpan *x_span;
   CoglTexSliceSpan *y_span;
-  gulong            enable_flags = (COGL_ENABLE_TEXTURE_2D
-                                   | COGL_ENABLE_VERTEX_ARRAY
-                                   | COGL_ENABLE_TEXCOORD_ARRAY);
 
 #if COGL_DEBUG
   printf("=== Drawing Tex Quad (Hardware Tiling Mode) ===\n");
 #endif
-  
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
-  /* Prepare GL state */
-  if (ctx->color_alpha < 255
-      || tex->bitmap.format & COGL_A_BIT)
-    {
-      enable_flags |= COGL_ENABLE_BLEND;
-    }
-  
-  cogl_enable (enable_flags);
-  
-  GE( cogl_wrap_glTexCoordPointer (2, GL_FIXED, 0, tex_coords) );
-  GE( cogl_wrap_glVertexPointer   (2, GL_FIXED, 0, quad_coords) );
-  
+
   /* Pick and bind opengl texture object */
   gl_handle = g_array_index (tex->slice_gl_handles, GLuint, 0);
-  GE( cogl_gles2_wrapper_bind_texture (tex->gl_target, gl_handle,
-				       tex->gl_intformat) );
-  
+
+  /* If we're using a different texture from the one already queued
+     then flush the vertices */
+  if (ctx->texture_vertices->len > 0
+      && gl_handle != ctx->texture_current)
+    _cogl_texture_flush_vertices ();
+  ctx->texture_target = tex->gl_target;
+  ctx->texture_current = gl_handle;
+  ctx->texture_format = tex->gl_intformat;
+
   /* Don't include the waste in the texture coordinates */
   x_span = &g_array_index (tex->slice_x_spans, CoglTexSliceSpan, 0);
   y_span = &g_array_index (tex->slice_y_spans, CoglTexSliceSpan, 0);
@@ -1910,18 +2177,75 @@ _cogl_texture_quad_hw (CoglTexture *tex,
   ty1 = ty1 * (y_span->size - y_span->waste) / y_span->size;
   ty2 = ty2 * (y_span->size - y_span->waste) / y_span->size;
 
-  /* Draw textured quad */
-  tex_coords[0] = tx1; tex_coords[1] = ty1;
-  tex_coords[2] = tx2; tex_coords[3] = ty1;
-  tex_coords[4] = tx1; tex_coords[5] = ty2;
-  tex_coords[6] = tx2; tex_coords[7] = ty2;
+  _cogl_texture_add_quad_vertices (CLUTTER_FIXED_TO_FLOAT (x1),
+                                   CLUTTER_FIXED_TO_FLOAT (y1),
+                                   CLUTTER_FIXED_TO_FLOAT (x2),
+                                   CLUTTER_FIXED_TO_FLOAT (y2),
+                                   CLUTTER_FIXED_TO_FLOAT (tx1),
+                                   CLUTTER_FIXED_TO_FLOAT (ty1),
+                                   CLUTTER_FIXED_TO_FLOAT (tx2),
+                                   CLUTTER_FIXED_TO_FLOAT (ty2));
+}
+
+void
+cogl_texture_multiple_rectangles (CoglHandle          handle,
+                                  const ClutterFixed *verts,
+                                  guint               n_rects)
+{
+  CoglTexture    *tex;
+  gulong          enable_flags = (COGL_ENABLE_VERTEX_ARRAY
+                                  | COGL_ENABLE_TEXCOORD_ARRAY
+                                  | COGL_ENABLE_TEXTURE_2D);
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* Check if valid texture */
+  if (!cogl_is_texture (handle))
+    return;
   
-  quad_coords[0] = x1; quad_coords[1] = y1;
-  quad_coords[2] = x2; quad_coords[3] = y1;
-  quad_coords[4] = x1; quad_coords[5] = y2;
-  quad_coords[6] = x2; quad_coords[7] = y2;
+  tex = _cogl_texture_pointer_from_handle (handle);
   
-  GE (cogl_wrap_glDrawArrays (GL_TRIANGLE_STRIP, 0, 4) );
+  /* Make sure we got stuff to draw */
+  if (tex->slice_gl_handles == NULL)
+    return;
+  
+  if (tex->slice_gl_handles->len == 0)
+    return;
+
+  /* Prepare GL state */
+  if (ctx->color_alpha < 255
+      || tex->bitmap.format & COGL_A_BIT)
+    enable_flags |= COGL_ENABLE_BLEND;
+
+  cogl_enable (enable_flags);
+
+  g_array_set_size (ctx->texture_vertices, 0);
+
+  while (n_rects-- > 0)
+    {
+      if (verts[4] != verts[6] && verts[5] != verts[7])
+        {
+          /* If there is only one GL texture and either the texture is
+             NPOT (no waste) or all of the coordinates are in the
+             range [0,1] then we can use hardware tiling */
+          if (tex->slice_gl_handles->len == 1
+              && ((cogl_features_available (COGL_FEATURE_TEXTURE_NPOT)
+                   && tex->gl_target == GL_TEXTURE_2D)
+                  || (verts[4] >= 0 && verts[4] <= CFX_ONE
+                      && verts[6] >= 0 && verts[6] <= CFX_ONE
+                      && verts[5] >= 0 && verts[5] <= CFX_ONE
+                      && verts[7] >= 0 && verts[7] <= CFX_ONE)))
+            _cogl_texture_quad_hw (tex, verts[0],verts[1], verts[2],verts[3],
+                                   verts[4],verts[5], verts[6],verts[7]);
+          else
+            _cogl_texture_quad_sw (tex, verts[0],verts[1], verts[2],verts[3],
+                                   verts[4],verts[5], verts[6],verts[7]);
+        }
+
+      verts += 8;
+    }
+
+  _cogl_texture_flush_vertices ();
 }
 
 void
@@ -1935,66 +2259,18 @@ cogl_texture_rectangle (CoglHandle   handle,
 			ClutterFixed tx2,
 			ClutterFixed ty2)
 {
-  CoglTexture       *tex;
-  ClutterFixed       tempx;
-  
-  /* Check if valid texture */
-  if (!cogl_is_texture (handle))
-    return;
-  
-  tex = _cogl_texture_pointer_from_handle (handle);
-  
-  /* Make sure we got stuff to draw */
-  if (tex->slice_gl_handles == NULL)
-    return;
-  
-  if (tex->slice_gl_handles->len == 0)
-    return;
-  
-  if (tx1 == tx2 || ty1 == ty2)
-    return;
-  
-  /* Fix quad coord ordering */
-  if (x1 > x2)
-    {
-      tempx = x1;
-      x1 = x2;
-      x2 = tempx;
-    }
-  
-  if (y1 > y2)
-    {
-      tempx = y1;
-      y1 = y2;
-      y2 = tempx;
-    }
-  
-  /* Fix texture coord ordering */
-  if (tx1 > tx2)
-    {
-      tempx = tx1;
-      tx1 = tx2;
-      tx2 = tempx;
-    }
-  
-  if (ty1 > ty2)
-    {
-      tempx = ty1;
-      ty1 = ty2;
-      ty2 = tempx;
-    }
-  
-  /* Tile textured quads */
-  if (tex->slice_gl_handles->len == 1
-      && tx1 >= -CFX_ONE && tx2 <= CFX_ONE
-      && ty1 >= -CFX_ONE && ty2 <= CFX_ONE)
-    {
-      _cogl_texture_quad_hw (tex, x1,y1, x2,y2, tx1,ty1, tx2,ty2);
-    }
-  else
-    {
-      _cogl_texture_quad_sw (tex, x1,y1, x2,y2, tx1,ty1, tx2,ty2);
-    }
+  ClutterFixed verts[8];
+
+  verts[0] = x1;
+  verts[1] = y1;
+  verts[2] = x2;
+  verts[3] = y2;
+  verts[4] = tx1;
+  verts[5] = ty1;
+  verts[6] = tx2;
+  verts[7] = ty2;
+
+  cogl_texture_multiple_rectangles (handle, verts, 1);
 }
 
 void
@@ -2039,27 +2315,9 @@ cogl_texture_polygon (CoglHandle         handle,
   /* Make sure there is enough space in the global texture vertex
      array. This is used so we can render the polygon with a single
      call to OpenGL but still support any number of vertices */
-  if (ctx->texture_vertices_size < n_vertices)
-    {
-      guint nsize = ctx->texture_vertices_size;
-      
-      if (nsize == 0)
-	nsize = 1;
-      do
-	nsize *= 2;
-      while (nsize < n_vertices);
-      
-      ctx->texture_vertices_size = nsize;
+  g_array_set_size (ctx->texture_vertices, n_vertices);
+  p = (CoglTextureGLVertex *) ctx->texture_vertices->data;
 
-      if (ctx->texture_vertices)
-	ctx->texture_vertices = g_realloc (ctx->texture_vertices,
-					   nsize
-					   * sizeof (CoglTextureGLVertex));
-      else
-	ctx->texture_vertices = g_malloc (nsize
-					  * sizeof (CoglTextureGLVertex));
-    }
-  
   /* Prepare GL state */
   enable_flags = (COGL_ENABLE_TEXTURE_2D
 		  | COGL_ENABLE_VERTEX_ARRAY
@@ -2082,14 +2340,14 @@ cogl_texture_polygon (CoglHandle         handle,
   if (use_color)
     {
       enable_flags |= COGL_ENABLE_COLOR_ARRAY;
-      GE( cogl_wrap_glColorPointer (4, GL_FIXED, sizeof (CoglTextureGLVertex),
-				    ctx->texture_vertices[0].c) );
+      GE( cogl_wrap_glColorPointer (4, GL_FLOAT,
+                                    sizeof (CoglTextureGLVertex), p->c) );
     }
- 
-  GE( cogl_wrap_glVertexPointer (3, GL_FIXED, sizeof (CoglTextureGLVertex),
-				 ctx->texture_vertices[0].v) );
-  GE( cogl_wrap_glTexCoordPointer (2, GL_FIXED, sizeof (CoglTextureGLVertex),
-				   ctx->texture_vertices[0].t) );
+
+  GE( cogl_wrap_glVertexPointer (3, GL_FLOAT,
+                                 sizeof (CoglTextureGLVertex), p->v ) );
+  GE( cogl_wrap_glTexCoordPointer (2, GL_FLOAT,
+                                   sizeof (CoglTextureGLVertex), p->t ) );
 
   cogl_enable (enable_flags);
   
@@ -2099,17 +2357,23 @@ cogl_texture_polygon (CoglHandle         handle,
 
   /* Convert the vertices into an array of GLfixeds ready to pass to
      OpenGL */
-  for (i = 0, p = ctx->texture_vertices; i < n_vertices; i++, p++)
+  for (i = 0; i < n_vertices; i++, p++)
     {
-      p->v[0] = vertices[i].x;
-      p->v[1] = vertices[i].y;
-      p->v[2] = vertices[i].z;
-      p->t[0] = vertices[i].tx * (x_span->size - x_span->waste) / x_span->size;
-      p->t[1] = vertices[i].ty * (y_span->size - y_span->waste) / y_span->size;
-      p->c[0] = (vertices[i].color.red << 16) / 0xff;
-      p->c[1] = (vertices[i].color.green << 16) / 0xff;
-      p->c[2] = (vertices[i].color.blue << 16) / 0xff;
-      p->c[3] = (vertices[i].color.alpha << 16) / 0xff;
+#define CFX_F CLUTTER_FIXED_TO_FLOAT
+
+      p->v[0] = CFX_F (vertices[i].x);
+      p->v[1] = CFX_F (vertices[i].y);
+      p->v[2] = CFX_F (vertices[i].z);
+      p->t[0] = CFX_F (vertices[i].tx
+                       * (x_span->size - x_span->waste) / x_span->size);
+      p->t[1] = CFX_F (vertices[i].ty
+                       * (y_span->size - y_span->waste) / y_span->size);
+      p->c[0] = vertices[i].color.red / 255.0f;
+      p->c[1] = vertices[i].color.green / 255.0f;
+      p->c[2] = vertices[i].color.blue / 255.0f;
+      p->c[3] = vertices[i].color.alpha / 255.0f;
+
+#undef CFX_F
     }
 
   GE( cogl_gles2_wrapper_bind_texture (tex->gl_target, gl_handle,
