@@ -17,7 +17,6 @@ typedef struct SuperOH
 {
   ClutterActor   **hand, *bgtex;
   ClutterActor    *group;
-  GdkPixbuf       *bgpixb;
 
 } SuperOH;
 
@@ -41,10 +40,12 @@ get_radius (void)
 
 /* input handler */
 static gboolean
-input_cb (ClutterStage *stage,
+input_cb (ClutterActor *stage,
 	  ClutterEvent *event,
 	  gpointer      data)
 {
+  SuperOH *oh = data;
+
   if (event->type == CLUTTER_BUTTON_PRESS)
     {
       ClutterButtonEvent *button_event;
@@ -57,7 +58,7 @@ input_cb (ClutterStage *stage,
       g_print ("*** button press event (button:%d) ***\n",
 	       button_event->button);
 
-      e = clutter_stage_get_actor_at_pos (stage, x, y);
+      e = clutter_stage_get_actor_at_pos (CLUTTER_STAGE (stage), x, y);
 
       if (e && (CLUTTER_IS_TEXTURE (e) || CLUTTER_IS_CLONE_TEXTURE (e)))
         {
@@ -77,6 +78,15 @@ input_cb (ClutterStage *stage,
 	  clutter_main_quit ();
           return TRUE;
         }
+      else if (clutter_key_event_symbol (kev) == CLUTTER_r)
+        {
+          gint i;
+
+          for (i = 0; i < n_hands; i++)
+            clutter_actor_show (oh->hand[i]);
+
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -89,8 +99,8 @@ frame_cb (ClutterTimeline *timeline,
 	  gint             frame_num,
 	  gpointer         data)
 {
-  SuperOH        *oh = (SuperOH *)data;
-  gint            i;
+  SuperOH *oh = data;
+  gint     i;
 
   /* Rotate everything clockwise about stage center*/
 
@@ -126,7 +136,6 @@ main (int argc, char *argv[])
   ClutterBehaviour *scaler_1, *scaler_2;
   ClutterActor    *stage;
   ClutterColor     stage_color = { 0x61, 0x64, 0x8c, 0xff };
-  GdkPixbuf       *pixbuf;
   SuperOH         *oh;
   gint             i;
   GError          *error;
@@ -150,11 +159,6 @@ main (int argc, char *argv[])
   stage = clutter_stage_get_default ();
   clutter_actor_set_size (stage, 800, 600);
 
-  pixbuf = gdk_pixbuf_new_from_file ("redhand.png", NULL);
-
-  if (!pixbuf)
-    g_error("pixbuf load failed");
-
   clutter_stage_set_title (CLUTTER_STAGE (stage), "Actors Test");
   clutter_stage_set_color (CLUTTER_STAGE (stage),
 		           &stage_color);
@@ -188,9 +192,16 @@ main (int argc, char *argv[])
       gint x, y, w, h;
       gint radius = get_radius ();
 
-      /* Create a texture from pixbuf, then clone in to same resources */
+      /* Create a texture from file, then clone in to same resources */
       if (i == 0)
-	oh->hand[i] = clutter_texture_new_from_pixbuf (pixbuf);
+	{
+	  if ((oh->hand[i] = clutter_texture_new_from_file ("redhand.png",
+							    &error)) == NULL)
+	    {
+	      g_error ("image load failed: %s", error->message);
+	      exit (1);
+	    }
+	}
       else
 	oh->hand[i] = clutter_clone_texture_new (CLUTTER_TEXTURE(oh->hand[0]));
 
@@ -224,14 +235,12 @@ main (int argc, char *argv[])
 #endif
     }
 
-  clutter_actor_show_all (oh->group);
-
   /* Add the group to the stage */
   clutter_container_add_actor (CLUTTER_CONTAINER (stage),
                                CLUTTER_ACTOR (oh->group));
 
   /* Show everying ( and map window ) */
-  clutter_actor_show_all (stage);
+  clutter_actor_show (stage);
 
 
   g_signal_connect (stage, "button-press-event",
